/*
 * C
 *
 * Copyright 2018-2024 MicroEJ Corp. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be found with this software.
 */

/**
 * @file
 * @brief LLGPIO Exercise implementation.
 * @author MicroEJ Developer Team
 * @version 1.0.0
 */

#include <stdint.h>
#include <stdio.h>
#include "sni.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "LLGPIO_impl.h"

#include "board.h"
#include "fsl_gpio.h"
#include "fsl_common.h"
#include "fsl_iomuxc.h"
#include "fsl_adapter_gpio.h"
#include "fsl_debug_console.h"

#ifdef __cplusplus
	extern "C" {
#endif


/*******************************************************************************
 * Definitions
 ******************************************************************************/

#define EXAMPLE_LED_GPIO_PORT     	BOARD_USER_LED_GPIO
#define EXAMPLE_LED_GPIO_PIN 		BOARD_USER_LED_GPIO_PIN

#define EXAMPLE_BUTTON_GPIO_PORT	13
#define EXAMPLE_BUTTON_GPIO_PIN     0
#define EXAMPLE_BUTTON_IRQ          BOARD_USER_BUTTON_IRQ

GPIO_HANDLE_DEFINE(s_ButtonEventHandle);

static uint8_t GPIO_initialized = 0;
static volatile jint button_index;

static SemaphoreHandle_t button_semaphore;

/** Interrupt request handler called when the button is pressed. */
void BOARD_ButtonHandler(void* arg)
{
	portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;
    xSemaphoreGiveFromISR(button_semaphore, &xHigherPriorityTaskWoken);
    if(xHigherPriorityTaskWoken != pdFALSE )
    {
        // Force a context switch here.
        portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
    }
}

static void LLGPIO_initialize(void)
{
	if(!GPIO_initialized)
	{
		GPIO_initialized = 1;

		// LED initialization

		/* GPIO configuration on GPIO_AD_04 (pin M13) */
		gpio_pin_config_t gpio9_pinM13_config = {
		  .direction = kGPIO_DigitalOutput,
		  .outputLogic = 0U,
		  .interruptMode = kGPIO_NoIntmode
		};
		/* Initialize GPIO functionality on GPIO_AD_04 (pin M13) */
		GPIO_PinInit(EXAMPLE_LED_GPIO_PORT, EXAMPLE_LED_GPIO_PIN, &gpio9_pinM13_config);

		IOMUXC_SetPinMux(
		  IOMUXC_GPIO_AD_04_GPIO9_IO03,           /* GPIO_AD_04 is configured as GPIO9_IO03 */
		  0U);

		// Button Initialization

		hal_gpio_pin_config_t buttonirq_sw_config = {
				kHAL_GpioDirectionIn,
				0,
				EXAMPLE_BUTTON_GPIO_PORT,
				EXAMPLE_BUTTON_GPIO_PIN
		};


		HAL_GpioInit(s_ButtonEventHandle, &buttonirq_sw_config);	
		HAL_GpioSetTriggerMode(s_ButtonEventHandle, kHAL_GpioInterruptRisingEdge);
		HAL_GpioInstallCallback(s_ButtonEventHandle, BOARD_ButtonHandler, NULL);
		EnableIRQ(EXAMPLE_BUTTON_IRQ);

		button_semaphore = xSemaphoreCreateBinary();
		xSemaphoreTake(button_semaphore, 0);
	}
}

void LLGPIO_set(int32_t pin, uint8_t state)
{	 
	LLGPIO_initialize();

	if( state == JFALSE)
	{
		GPIO_PinWrite(EXAMPLE_LED_GPIO_PORT, EXAMPLE_LED_GPIO_PIN, 0U);
	}
	else
	{
		GPIO_PinWrite(EXAMPLE_LED_GPIO_PORT, EXAMPLE_LED_GPIO_PIN, 1U);
	}
}

uint8_t LLGPIO_get(int32_t pin)
{
    LLGPIO_initialize();
    return (GPIO_ReadPinInput(EXAMPLE_LED_GPIO_PORT, EXAMPLE_LED_GPIO_PIN))  == 0 ? JFALSE : JTRUE;

}

jint Java_com_microej_training_gpio_example_GpioExercise_waitButton()
{  
  // Initialize the GPIOs
  LLGPIO_initialize();
  
  // Wait for the interrupt
  xSemaphoreTake(button_semaphore, portMAX_DELAY);
  return button_index;
}

#ifdef __cplusplus
	}
#endif
