/*
 * C
 *
 * Copyright 2018-2023 MicroEJ Corp. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be found with this software.
 */

/**
 * @file
 * @brief LLGPIO Exercise implementation.
 * @author MicroEJ Developer Team
 * @version 1.0.0
 * @date 3 March 2023
 */

#include <stdint.h>
#include <stdio.h>
#include "sni.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "LLGPIO_impl.h"
#include "hal_stm32.h"
#include "stm32f7508_discovery.h"

#ifdef __cplusplus
	extern "C" {
#endif

static uint8_t GPIO_initialized = 0;
static jint button_index = 0;

static xSemaphoreHandle button_semaphore;

static void LLGPIO_initialize(void)
{
    if(!GPIO_initialized)
    {
      GPIO_initialized = 1;

      /* LED Init */
      __GPIOI_CLK_ENABLE();
      GPIO_InitTypeDef GPIO_InitStruct;

      /* Configure LED pin as output */
      GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
      GPIO_InitStruct.Pull = GPIO_NOPULL;
      GPIO_InitStruct.Speed = GPIO_SPEED_LOW;
      GPIO_InitStruct.Pin = GPIO_PIN_1;

      HAL_GPIO_Init(GPIOI, &GPIO_InitStruct);

      /* Enable the BUTTON Clock */
      BUTTONx_GPIO_CLK_ENABLE(BUTTON_WAKEUP);

      /* Configure Button pin as input with external interrupt */
      GPIO_InitStruct.Pull = GPIO_NOPULL;
      GPIO_InitStruct.Speed = GPIO_SPEED_FAST;
      GPIO_InitStruct.Pin = WAKEUP_BUTTON_PIN;
      GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
      HAL_GPIO_Init(WAKEUP_BUTTON_GPIO_PORT, &GPIO_InitStruct);

      /* Configure Button EXTI Interrupt to the lowest priority */
      HAL_NVIC_SetPriority(WAKEUP_BUTTON_EXTI_IRQn, 0xf, 0);
      HAL_NVIC_EnableIRQ((IRQn_Type)(WAKEUP_BUTTON_EXTI_IRQn));

      vSemaphoreCreateBinary(button_semaphore);  
      xSemaphoreTake(button_semaphore, 0);
    }
}

void LLGPIO_set(int32_t pin, uint8_t state)
{	 
    GPIO_PinState value;
    LLGPIO_initialize();

	if(state == JFALSE)
	{
		value = GPIO_PIN_RESET;
	}
    else
	{
    	value = GPIO_PIN_SET;
	}

    HAL_GPIO_WritePin(GPIOI, GPIO_PIN_1, value);
}

uint8_t LLGPIO_get(int32_t pin)
{
    LLGPIO_initialize();
    GPIO_PinState state = HAL_GPIO_ReadPin(GPIOI, GPIO_PIN_1);
    return (state == GPIO_PIN_RESET ? JFALSE : JTRUE);

}

jint Java_com_microej_training_gpio_example_GpioExercise_waitButton()
{  
  // Initialize the GPIOs
  LLGPIO_initialize();
  
  // Wait for the interrupt
  xSemaphoreTake(button_semaphore, portMAX_DELAY);
  return button_index;
}

/** Interrupt request handler called when the button is pressed. */

void EXTI15_10_IRQHandler(void)
{
    if(__HAL_GPIO_EXTI_GET_IT(WAKEUP_BUTTON_PIN) != RESET)
    {
		if (HAL_GPIO_ReadPin(WAKEUP_BUTTON_GPIO_PORT, WAKEUP_BUTTON_PIN) == GPIO_PIN_SET)
		{
                        button_index = (jint)WAKEUP_BUTTON_PIN; // WAKEUP_BUTTON_PIN = 0x800
                        
			// send an event to wake up the IOE interrupt management task            
			portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE; 
			xSemaphoreGiveFromISR(button_semaphore, &xHigherPriorityTaskWoken);
			
			/* Switch tasks if necessary. */
			if(xHigherPriorityTaskWoken != pdFALSE)
			{
                          portEND_SWITCHING_ISR(xHigherPriorityTaskWoken);
			}
		}      
                __HAL_GPIO_EXTI_CLEAR_IT(WAKEUP_BUTTON_PIN);
    }
}
  
#ifdef __cplusplus
	}
#endif
